//=============================================================================
// AM_Pickup.
//
// Author: Francesco Biscazzo
// Date: 2016
//=============================================================================
class AM_Pickup extends TournamentPickup;

var AdvancedMutator AM;
var Weapon OwnerWeapon;

var ZoneInfo oldOwnerZone;
var String oldOwnerState;
var String oldOwnerWeaponState;
var EPhysics oldOwnerPhysics;

function RegisterAdvancedMutator(AdvancedMutator AdvMut) {
	AM = AdvMut;
	SetCollision(true, false, false);
	bCollideWorld = true;
	SetCollisionSize( Owner.CollisionRadius, Owner.CollisionHeight );
	
	GotoState('Activated');
	Activate();
	
	SetTimer(AM.callbackRate, True);
}

state Activated
{
	function Activate() {
		bActive = true;
		SetOwnerWeapon();
	}

	function EndState()
	{
		bActive = false;
		GotoState('Idle2');
	}

	function BeginState()
	{
		bActive = true;
		SetOwnerWeapon();	
	}
	
	// Use getStateName() on the weapon to get the fire type.
	function FireEffect() {
		AM.Fired(Pawn(Owner), OwnerWeapon, AM.static.getAimedActor(Pawn(Owner), OwnerWeapon));
	}

	function OwnerJumped() {
		AM.Jumped(Pawn(Owner));
	}

	function ChangedWeapon()
	{
		local Weapon OldWeapon;

		OldWeapon = OwnerWeapon;
		super.ChangedWeapon();
		SetOwnerWeapon();
		
		AM.ChangedWeapon(Pawn(Owner), OldWeapon, OwnerWeapon);
	}

	function bool HandlePickupQuery( Inventory item )
	{
		AM.TouchedPickUp(Pawn(Owner), item);
	}
	
	function ZoneChange( ZoneInfo newZone ) {
		AM.ChangedZone(Pawn(Owner), oldOwnerZone, newZone);
		oldOwnerZone = newZone;
	}

	function SetOwnerWeapon()
	{
		if ( !bActive )
			return;

		// Make old weapon normal again.
		if ( OwnerWeapon != None )
		{
			OwnerWeapon.SetDefaultDisplayProperties();
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = None;
		}
			
		OwnerWeapon = Pawn(Owner).Weapon;
		// Set new weapon as the OwnerWeapon.
		if ( OwnerWeapon != None )
		{
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = self;
		}
	}
	
	function HitWall( vector HitNormal, actor HitWall ) {
		AM.HittingWall(Pawn(Owner), HitNormal, HitWall);
	}
	
	function Falling() {
		AM.Fell(Pawn(Owner));
	}
	
	function Landed( vector HitNormal ) {
		AM.Landing(Pawn(Owner), HitNormal);
	}
	
	function Touch( Actor Other ) {
		AM.Touching(Pawn(Owner), Other);
	}
	
	function PostTouch( Actor Other ) {
		AM.Touched(Pawn(Owner), Other);
	}
	
	function UnTouch( Actor Other ) {
		AM.UnTouched(Pawn(Owner), Other);
	}
	
	function Bump( Actor Other ) {
		AM.Bumped(Pawn(Owner), Other);
	}
	
	function BaseChange() {
		AM.ChangedBase(Pawn(Owner));
	}
	
	function EncroachedBy( actor Other ) {
		AM.Encroached(Pawn(Owner), Other);
	}
	
	function EndedRotation() {
		AM.EndingRotation(Pawn(Owner));
	}

	function FellOutOfWorld() {}

	function Timer()
	{
		local String currentOwnerState;
		local String currentOwnerWeaponState;
		local EPhysics currentOwnerPhysics;
	
		SetLocation(Owner.Location);
		SetRotation(Owner.Rotation);
	
		if (Owner.IsA('Pawn'))
				Pawn(Owner).bCountJumps = true;

		if ( !Pawn(Owner).bAutoActivate )
			OwnerJumped();
	
		currentOwnerState = String(Owner.getStateName());
		if (currentOwnerState != oldOwnerState) {
			AM.ChangedState(Pawn(Owner), oldOwnerState, currentOwnerState);
			oldOwnerState = currentOwnerState;
		}
		
		currentOwnerWeaponState = String(OwnerWeapon.getStateName());
		if (currentOwnerWeaponState != oldOwnerWeaponState) {
			AM.ChangedWeaponState(Pawn(Owner), OwnerWeapon, oldOwnerWeaponState, currentOwnerWeaponState);
			oldOwnerWeaponState = currentOwnerWeaponState;
		}
		
		currentOwnerPhysics = Owner.Physics;
		if (currentOwnerPhysics != oldOwnerPhysics) {
			AM.ChangedPhysics(Pawn(Owner), oldOwnerPhysics, currentOwnerPhysics);
			oldOwnerPhysics = currentOwnerPhysics;
			
			SetPhysics(currentOwnerPhysics);
		}
		
		AM.AimedActor(Pawn(Owner), OwnerWeapon, AM.static.getAimedActor(Pawn(Owner), OwnerWeapon));
		
		AM.EveryTimeCallback(self);
	}
}